<?php

declare(strict_types=1);

namespace Drupal\rdf_sync\Drush\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Consolidation\AnnotatedCommand\Hooks\HookManager;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\rdf_sync\Model\SyncMethod;
use Drupal\rdf_sync\RdfSyncMapperInterface;
use Drupal\rdf_sync\RdfSyncSynchronizer;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Drush\Utils\StringUtils;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Drush commands used to synchronize entities with the RDF backend.
 */
class RdfSyncCommands extends DrushCommands {

  public function __construct(
    protected RdfSyncSynchronizer $synchronizer,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    protected RdfSyncMapperInterface $mapper,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('rdf_sync.synchronizer'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('rdf_sync.mapper'),
    );
  }

  /**
   * Enable synchronization.
   */
  #[CLI\Command(name: 'rdf_sync:enable')]
  public function enableSynchronization(): void {
    $this->synchronizer->enableSynchronization();
    $this->logger()->notice(dt('Enabled synchronization'));
  }

  /**
   * Disable synchronization.
   */
  #[CLI\Command(name: 'rdf_sync:disable')]
  public function disableSynchronization(): void {
    $this->synchronizer->disableSynchronization();
    $this->logger()->notice(dt('Disabled synchronization'));
  }

  /**
   * Synchronize the RDF backend for a given content type.
   */
  #[CLI\Command(name: 'rdf_sync:synchronize')]
  #[CLI\Argument(name: 'entityTypeId', description: 'The entity type (e.g., node, taxonomy_term')]
  #[CLI\Option(name: 'bundle', description: 'Comma delimited list of bundles to limit on')]
  public function synchronize(
    string $entityTypeId,
    array $options = ['bundle' => self::REQ],
  ): void {
    $entityType = $this->entityTypeManager->getDefinition($entityTypeId);

    $query = $this->entityTypeManager->getStorage($entityTypeId)->getQuery()->accessCheck(FALSE);

    if ($entityType->hasKey('bundle')) {
      $bundleKey = $entityType->getKey('bundle');
      $query->condition($bundleKey, $options['bundle'], 'IN');
    }

    // Only sync published content.
    if ($entityType->entityClassImplements(EntityPublishedInterface::class) && $entityType->hasKey('published')) {
      $publishedKey = $entityType->getKey('published');
      $query->condition($publishedKey, TRUE);
    }
    $ids = array_values($query->execute());

    $batchBuilder = (new BatchBuilder())->addOperation([static::class, 'init'], [count($ids)]);
    foreach (array_chunk($ids, 5) as $queuedIds) {
      $batchBuilder->addOperation([static::class, 'synchronizeBatch'], [$entityTypeId, $queuedIds]);
    }
    batch_set($batchBuilder->toArray());
    drush_backend_batch_process();
  }

  /**
   * Validates the rdf_sync:synchronize command argument and options.
   */
  #[CLI\Hook(type: HookManager::ARGUMENT_VALIDATOR, target: 'rdf_sync:synchronize')]
  public function validateSynchronize(CommandData $commandData): void {
    $entityTypeId = $commandData->input()->getArgument('entityTypeId');
    if (!$this->entityTypeManager->getDefinition($entityTypeId, FALSE)) {
      throw new \InvalidArgumentException('Invalid entity type ID');
    }

    $passedBundles = StringUtils::csvToArray($commandData->input()->getOption('bundle'));
    $availableBundles = array_keys($this->entityTypeBundleInfo->getBundleInfo($entityTypeId));

    // Check for invalid passed bundles.
    if ($diff = array_diff($passedBundles, $availableBundles)) {
      throw new \InvalidArgumentException('Invalid bundles: ' . implode(', ', $diff));
    }

    // Check for unmapped passed bundles.
    $unmappedBundles = array_filter($passedBundles, function (string $bundle) use ($entityTypeId): bool {
      return !$this->mapper->isMappedBundle($entityTypeId, $bundle);
    });
    if ($unmappedBundles) {
      throw new \InvalidArgumentException('Unmapped bundles: ' . implode(', ', $unmappedBundles));
    }

    $bundlesToProcess = $passedBundles;
    if (!$bundlesToProcess) {
      // If no bundles were passed, get all mapped bundles.
      $bundlesToProcess = array_filter($availableBundles, function (string $bundle) use ($entityTypeId): bool {
        return $this->mapper->isMappedBundle($entityTypeId, $bundle);
      });
      if (!$bundlesToProcess) {
        throw new \InvalidArgumentException("The '$entityTypeId' entity type has no mapped bundle");
      }
    }

    $commandData->input()->setOption('bundle', $bundlesToProcess);
  }

  /**
   * Initiates the batch process.
   *
   * @param int $total
   *   Total number of entities to be synchronized.
   * @param array $context
   *   The batch process context.
   */
  public static function init(int $total, array &$context): void {
    $context['results']['total'] = $total;
    $context['results']['progress'] = 0;
  }

  /**
   * Synchronizes one batch.
   *
   * @param string $entityTypeId
   *   The entity type ID of the entities being synchronized.
   * @param int[]|string[] $ids
   *   A list of entity IDs.
   * @param array $context
   *   The batch process context.
   */
  public static function synchronizeBatch(string $entityTypeId, array $ids, array &$context): void {
    $synchronizer = \Drupal::getContainer()->get('rdf_sync.synchronizer');
    $storage = \Drupal::entityTypeManager()->getStorage($entityTypeId);
    /** @var \Drupal\Core\Entity\ContentEntityInterface[] $entities */
    $entities = $storage->loadMultiple($ids);
    $synchronizer->synchronize(SyncMethod::UPDATE, $entities, TRUE);
    $synchronizer->destruct();
    $context['results']['progress'] += count($entities);
    $context['message'] = "Synchronized {$context['results']['progress']} out of {$context['results']['total']} entities";
  }

}
